"""Module for working with Service Area Solver."""

from enum import IntEnum
from arcgisscripting import na as cna  # pylint:disable=no-name-in-module

# List of names exported from this module
__all__ = ["ServiceAreaInputDataType", "ServiceAreaOutputDataType", "ServiceAreaPolygonCutoffGeometry",
           "ServiceAreaOverlapGeometry", "ServiceAreaPolygonDetail", "ServiceAreaOutputType", "ServiceArea"]


class ServiceAreaInputDataType(IntEnum):
    """Enumeration for feature class names used by the Service Area solver to store inputs."""

    Facilities = 0
    PointBarriers = 1
    LineBarriers = 2
    PolygonBarriers = 3


class ServiceAreaOutputDataType(IntEnum):
    """Enumeration for feature class names used by the Service Area solver to store outputs."""

    Facilities = 0
    Lines = 1
    Polygons = 2


class ServiceAreaPolygonCutoffGeometry(IntEnum):
    """Enumeration for supported polygon cutoff geometry types."""

    Disks = 0
    Rings = 1


class ServiceAreaOverlapGeometry(IntEnum):
    """Enumeration for the behavior of service-area output from multiple facilities in relation to one another."""

    Split = 0
    Overlap = 1
    Dissolve = 2


class ServiceAreaPolygonDetail(IntEnum):
    """Enumeration for supported polygon detail options."""

    Generalized = 0
    Standard = 1
    High = 2


class ServiceAreaOutputType(IntEnum):
    """Enumeration for the type of output to be generated."""

    Polygons = 0
    Lines = 1
    PolygonsAndLines = 2


class ServiceArea(cna.ServiceArea):
    """Perform service area analysis."""

    pass
