"""Module to work with various Network Analyst Solvers."""

from enum import IntEnum

# List of names exported from this module
__all__ = ["MessageSeverity", "TimeZoneUsage", "TravelDirection", "TimeUnits", "DistanceUnits"]


class MessageSeverity(IntEnum):
    """Enumeration for severity level of solver messages."""

    Info = 0
    Warning = 1
    Error = 2
    WarningAndError = 3


class TimeZoneUsage(IntEnum):
    """Enumeration for specifying time zone for the time of day."""

    LocalTimeAtLocations = 0
    UTC = 1


class TravelDirection(IntEnum):
    """Enumeration to specify the direction of travel to or from the facilities."""

    FromFacility = 0
    ToFacility = 1


class TimeUnits(IntEnum):
    """Enumeration for time units."""

    Seconds = 20  # esriNAUSeconds
    Minutes = 21  # esriNAUMinutes
    Hours = 22  # esriNAUHours
    Days = 23  # esriNAUDays


class DistanceUnits(IntEnum):
    """Enumeration for distance units."""

    Feet = 3  # esriNAUFeet
    Yards = 4  # esriNAUYards
    Miles = 5  # esriNAUMiles
    NauticalMiles = 6  # esriNAUNauticalMiles
    Meters = 9  # esriNAUMeters
    Kilometers = 10  # esriNAUKilometers
