# -*- coding: utf-8 -*-
#COPYRIGHT 2018 ESRI
#
#TRADE SECRETS: ESRI PROPRIETARY AND CONFIDENTIAL
#Unpublished material - all rights reserved under the
#Copyright Laws of the United States.
#
#For additional information, contact:
#Environmental Systems Research Institute, Inc.
#Attn: Contracts Dept
#380 New York Street
#Redlands, California, USA 92373
#
#email: contracts@esri.com
r"""The ArcGIS Pro Aviation toolbox contains a powerful set of tools that
allows users to create, analyze, and manage aviation content."""
__all__ = ['AnalyzeRunwayObstacles', 'CreateCurvedApproach', 'FAA13A', 'FAA18B',
           'FAAFAR77', 'GenerateOISObstacleData', 'GenerateOISProfileData',
           'ICAOAnnex14', 'ICAOAnnex15', 'UnifiedFacilitiesCriteria']
__alias__ = 'aviation'
from arcpy.geoprocessing._base import gptooldoc, gp, gp_fixargs
from arcpy.arcobjects.arcobjectconversion import convertArcObjectToPythonObject

# Airports\Analysis toolset
@gptooldoc('AnalyzeRunwayObstacles_aviation', None)
def AnalyzeRunwayObstacles(input_ois_features=None, input_obstacle_features=None, out_feature_class=None, height_field=None, unit_field=None):
    """AnalyzeRunwayObstacles_aviation(input_ois_features, input_obstacle_features, out_feature_class, {height_field}, {unit_field})

        Examines Obstruction Identification Surfaces (OIS) with obstacle data
        to determine and analyze how far the obstacles intersect an OIS.

     INPUTS:
      input_ois_features (Feature Layer):
          The multipatch features with defined Airport schema. The feature
          class must be z-enabled.
      input_obstacle_features (Feature Layer):
          The input obstacle features that will be analyzed. The feature class
          must be z-enabled.
      height_field {String}:
          The field containing the height of the obstacle feature.
      unit_field {String}:
          The unit of the obstacle height.

     OUTPUTS:
      out_feature_class (Feature Class):
          A feature class containing one point for each obstacle feature that
          falls within the area covered by the input OIS. If the geometry type
          of the input obstacle feature is a polyline or polygon, a multipoint
          feature class will be created."""
    from arcpy.geoprocessing._base import gp, gp_fixargs
    from arcpy.arcobjects.arcobjectconversion import convertArcObjectToPythonObject
    try:
        retval = convertArcObjectToPythonObject(gp.AnalyzeRunwayObstacles_aviation(*gp_fixargs((input_ois_features, input_obstacle_features, out_feature_class, height_field, unit_field), True)))
        return retval
    except Exception as e:
        raise e

@gptooldoc('GenerateOISObstacleData_aviation', None)
def GenerateOISObstacleData(in_runway_features=None, in_dems=None, in_obstacle_features=None, target_ois_features=None, obstacle_height_field=None, obstacle_height_unit=None, in_flightpath_features=None):
    """GenerateOISObstacleData_aviation(in_runway_features, in_dems;in_dems..., in_obstacle_features, target_ois_features, {obstacle_height_field}, {obstacle_height_unit}, {in_flightpath_features})

        Enhances select Obstruction Identification Surface (OIS) features by
        determining the data necessary to build a skyline profile of point,
        polyline, or polygon features that may or may not be true obstacles to
        safe flight approach, adding collections of 3D points representing
        obstacle cross-sections to those features using JSON.

     INPUTS:
      in_runway_features (Feature Layer):
          This is the input runway dataset. The feature class must be z-enabled
          and contain polylines.
      in_dems (Raster Layer):
          The DEMs covering the runways and their approach obstruction
          identification surfaces.
      in_obstacle_features (Feature Layer):
          The input obstacle features that will be analyzed. The feature class
          must be z-enabled.
      target_ois_features (Feature Layer):
          The multipatch features with defined Airport schema. The feature
          class must be z-enabled.
      obstacle_height_field {String}:
          The field containing the height of the obstacle feature.
      obstacle_height_unit {String}:
          The unit of the obstacle height.
      in_flightpath_features {Feature Layer}:
          The polyline features that define curved approaches to the specified
          runways. If unspecified, all input features will be processed as
          straight approaches."""
    from arcpy.geoprocessing._base import gp, gp_fixargs
    from arcpy.arcobjects.arcobjectconversion import convertArcObjectToPythonObject
    try:
        retval = convertArcObjectToPythonObject(gp.GenerateOISObstacleData_aviation(*gp_fixargs((in_runway_features, in_dems, in_obstacle_features, target_ois_features, obstacle_height_field, obstacle_height_unit, in_flightpath_features), True)))
        return retval
    except Exception as e:
        raise e

@gptooldoc('GenerateOISProfileData_aviation', None)
def GenerateOISProfileData(in_runway_features=None, in_dems=None, target_ois_features=None, in_flightpath_features=None, sampling_distance=None, sample_profile_ois=None, sample_profile_runways=None):
    """GenerateOISProfileData_aviation(in_runway_features, in_dems;in_dems..., target_ois_features, {in_flightpath_features}, {sampling_distance}, {sample_profile_ois}, {sample_profile_runways})

        Generates a JSON string to represent various relationships. This tool
        enhances select Obstruction Identification Surfaces (OIS) features by
        determining the minimum, maximum, and true elevations along the
        approach to their runways, adding collections of 3D points
        representing those elevations to those features using JSON.

     INPUTS:
      in_runway_features (Feature Layer):
          This is the input runway dataset. The feature class must be z-enabled
          and contain polylines.
      in_dems (Raster Layer):
          The DEMs covering the runways and their approach obstruction
          identification surfaces.
      target_ois_features (Feature Layer):
          The multipatch features with defined Airport schema. The feature
          class must be z-enabled.
      in_flightpath_features {Feature Layer}:
          The polyline features that define curved approaches to the specified
          runways. If unspecified, all input features will be processed as
          straight approaches.
      sampling_distance {Double}:
          The distance, in meters, between generated points. The default is 30.
      sample_profile_ois {Boolean}:
          Specifies whether elevation points for OIS features will be generated.

          * PROFILE_OIS-Generates elevation points.

          * NO_PROFILE_OIS-Elevation points will not be generated. This is
          default.
      sample_profile_runways {Boolean}:
          Specifies whether elevation points along the runways will be
          generated.

          * PROFILE_RUNWAY-Generates elevation points along the runways.

          * NO_PROFILE_RUNWAY-Uses only the start and end points of the runways.
          This is default."""
    from arcpy.geoprocessing._base import gp, gp_fixargs
    from arcpy.arcobjects.arcobjectconversion import convertArcObjectToPythonObject
    try:
        retval = convertArcObjectToPythonObject(gp.GenerateOISProfileData_aviation(*gp_fixargs((in_runway_features, in_dems, target_ois_features, in_flightpath_features, sampling_distance, sample_profile_ois, sample_profile_runways), True)))
        return retval
    except Exception as e:
        raise e


# Airports\Obstruction Identification Surfaces toolset
@gptooldoc('CreateCurvedApproach_aviation', None)
def CreateCurvedApproach(in_runway_features=None, in_flightpath_features=None, target_ois_features=None, specification=None, runway_classification=None):
    """CreateCurvedApproach_aviation(in_runway_features, in_flightpath_features, target_ois_features, specification, runway_classification)

        Creates curved approach surfaces. This tool creates surfaces such as a
        polygon or multipatch features.

     INPUTS:
      in_runway_features (Feature Layer):
          This is the input runway dataset. The feature class must be z-enabled
          and contain polylines.
      in_flightpath_features (Feature Layer):
          The polyline features that define curved approaches to the specified
          runways.
      target_ois_features (Feature Layer):
          The target feature class that will contain the generated obstruction
          identification surfaces.
      specification (String):
          The specification of the approach surface.

          * ETOD-ICAO Annex 15 (ETOD)

          * OLS-ICAO Annex 14 (OLS)

          * FAR77-FAA regulations part 77 (FAR77)

          * 18B-FAA Advisory circular 150/5300_18B (18B)

          * 13A-FAA Advisory circular 150/5300_13A (13A)
      runway_classification (String):
          The runway classification of the approach surface.The options in the
          Input Specification drop-down will determine and
          correspond with the choices that appear in the Runway Classification
          parameter. For more information on the choices in the Specification
          drop-down see FAA 13A, FAA 18B, FAA FAR77, ICAO Annex 14 and ICAO
          Annex 15."""
    from arcpy.geoprocessing._base import gp, gp_fixargs
    from arcpy.arcobjects.arcobjectconversion import convertArcObjectToPythonObject
    try:
        retval = convertArcObjectToPythonObject(gp.CreateCurvedApproach_aviation(*gp_fixargs((in_runway_features, in_flightpath_features, target_ois_features, specification, runway_classification), True)))
        return retval
    except Exception as e:
        raise e

@gptooldoc('FAA13A_aviation', None)
def FAA13A(in_features=None, target=None, runway_type=None, highend_clear_way_length=None, lowend_clear_way_length=None):
    """FAA13A_aviation(in_features, target, runway_type, {highend_clear_way_length}, {lowend_clear_way_length})

        Creates obstacle identification surfaces (OIS) based on the FAA 13A
        specification. These surfaces describe airspaces that should be free
        of obstructions. Surfaces are used to support planning and design
        activities. The type, function, and dimension of a surface differ by
        runway classification.

     INPUTS:
      in_features (Feature Layer):
          The input runway dataset. The feature class must be z-enabled and
          contain polylines.
      target (Feature Layer):
          The target feature class that will contain the generated obstruction
          identification surfaces.
      runway_type (String):
          The classification of the runway.

          * SMALL_AIRPLANE_APPROACH_SPEEDS_LT_50-A runway designed for light
          aircraft with a maximum takeoff weight of less than 254 pounds and
          approach speed less than 50 knots. This is a visual runway only that
          can be used during the day or night.

          * SMALL_AIRPLANE_APPROACH_SPEEDS_GT_50-A runway designed for light
          aircraft with a maximum takeoff weight of less than 1,320 pounds and
          approach speed more than 50 knots. This is a visual runway only that
          can be used during the day or night.

          * LARGE_AIRPLANE_INSTRUMENT_GT_1_MILE_VISIBILITY-A runway designed
          for an aircraft with a maximum certified takeoff weight of more than
          12,500 pounds. The approach end of the runway is expected to serve
          large airplanes as a visual runway available during the day or night,
          or an instrument approach with a minimum greater than one statute mile
          (1.6 KM) only during the day.

          * INSTRUMENT_CATEGORY_A_AND_B-A runway designed for night operation
          with civil single-engine aircraft and twin-engine airplanes as well as
          those that reach 91 knots or greater but less than 121 knots. The
          approach end of the runway is expected to support instrument night
          operations and serve approach category A and B aircraft only.

          * INSTRUMENT_GT_CATEGORY_B-A runway designed for night operation with
          aircraft that can travel at speeds of 121 knots or greater. The
          approach end of the runway is expected to support instrument night
          operations serving greater than approach category B aircraft.

          * INSTRUMENT_GT_3/4_MILE_LT_1_MILE_VISIBILITY-A runway designed for
          an instrument approach procedure where the visibility is greater than
          three-fourths of a mile and less than one mile. The approach end of
          the runway is expected to accommodate instrument approaches with
          visible minimums more than three-fourths but less than 1 statute
          mile-more than 1.2 KM but less than 1.6 KM-during the day or night.

          * INSTRUMENT_LT_3/4_MILE_VISIBILITY-A runway where the course and
          vertical path guidance are provided with visibility less than three-
          fourths of a mile. The approach end of this runway is expected to
          accommodate instrument approaches with visibility minimum less than
          three-fourths of a statute mile (1.2 KM) or precision approach
          (Instrument Landing System [ILS] or Global Navigation Satellite System
          [GNSS] Landing System [GLS]) day or night.

          * VERTICAL_GUIDANCE_APPROACH-A runway that uses precision guidance
          systems to support aircraft approach and landing. The approach end of
          the runway is expected to accommodate approaches with vertical
          guidance such as a Glide Path Qualification Surface (GPQS).

          * DEPARTURE-A runway that allows pilots to follow standard departure
          procedures.
      highend_clear_way_length {Double}:
          The length of the area at the high end of the runway. The unit of
          measurement is based on the input runway features.
      lowend_clear_way_length {Double}:
          The length of the area at the low end of the runway. The unit of
          measurement is based on the input runway features."""
    from arcpy.geoprocessing._base import gp, gp_fixargs
    from arcpy.arcobjects.arcobjectconversion import convertArcObjectToPythonObject
    try:
        retval = convertArcObjectToPythonObject(gp.FAA13A_aviation(*gp_fixargs((in_features, target, runway_type, highend_clear_way_length, lowend_clear_way_length), True)))
        return retval
    except Exception as e:
        raise e

@gptooldoc('FAA18B_aviation', None)
def FAA18B(in_features=None, target=None, runway_type=None, highend_clear_way_length=None, lowend_clear_way_length=None, airport_elevation=None, include_merged_surface=None):
    """FAA18B_aviation(in_features, target, runway_type, {highend_clear_way_length}, {lowend_clear_way_length}, {airport_elevation}, {include_merged_surface})

        Creates obstruction identification surfaces based on the FAA 18B
        specification. These surfaces describe airspaces that should be free
        of obstructions. Surfaces are used to support planning and design
        activities. The type, function, and dimension of a surface differ by
        runway classification.

     INPUTS:
      in_features (Feature Layer):
          The input runway dataset. The feature class must be z-enabled and
          contain polylines.
      target (Feature Layer):
          The target feature class that will contain the generated obstacle
          identification surfaces.
      runway_type (String):
          The runway classification of in_features.

          * NON_VERTICAL_GUIDANCE_TYPE_1-A runway designed for visual maneuvers,
          nonvertically guided operations, and instrument departure procedures.

          * NON_VERTICAL_GUIDANCE_TYPE_2-A specially prepared hard surface
          (SPHS) runway designed for visual maneuvers, nonvertically guided
          operations, and instrument departure procedures. SPHS runways have a
          primary surface that extends 200 feet beyond each end of the runway.

          * VERTICAL_GUIDANCE-A runway that uses precision guidance systems to
          support aircraft approach and landing.
      highend_clear_way_length {Double}:
          The length of the area at the high end of the runway. The unit of
          measurement is based on the input runway features.
      lowend_clear_way_length {Double}:
          The length of the area at the low end of the runway. The unit of
          measurement is based on the input runway features.
      airport_elevation {Double}:
          The highest point on any runway in an airport. The unit of measurement
          is in meters. The default is the highest point from the runway end.
      include_merged_surface {Boolean}:
          Indicates whether merged horizontal and conical surfaces are included
          in the OIS in addition to the regular surfaces.

          * INCLUDE_MERGED_SURFACE-Include merged surfaces in the OIS output.
          This is the default.

          * NOT_INCLUDE_MERGED_SURFACE-Do not include merged surfaces in the
          OIS output."""
    from arcpy.geoprocessing._base import gp, gp_fixargs
    from arcpy.arcobjects.arcobjectconversion import convertArcObjectToPythonObject
    try:
        retval = convertArcObjectToPythonObject(gp.FAA18B_aviation(*gp_fixargs((in_features, target, runway_type, highend_clear_way_length, lowend_clear_way_length, airport_elevation, include_merged_surface), True)))
        return retval
    except Exception as e:
        raise e

@gptooldoc('FAAFAR77_aviation', None)
def FAAFAR77(in_features=None, target=None, high_runway_end_type=None, low_runway_end_type=None, specially_prepared_hard_surface_runway=None, highend_clear_way_length=None, lowend_clear_way_length=None, airport_elevation=None, include_merged_surface=None):
    """FAAFAR77_aviation(in_features, target, high_runway_end_type, low_runway_end_type, {specially_prepared_hard_surface_runway}, {highend_clear_way_length}, {lowend_clear_way_length}, {airport_elevation}, {include_merged_surface})

        Creates obstruction identification surfaces based on the FAA FAR 77
        specification. These surfaces describe airspace segments that are
        clear of obstructions. The type, function, and dimension of a surface
        differs by runway classification. This tool creates surfaces as
        polygon or multipatch features.

     INPUTS:
      in_features (Feature Layer):
          The input runway dataset. The feature class must be z-enabled and
          contain polylines.
      target (Feature Layer):
          The target feature class that will contain the generated obstruction
          identification surfaces.
      high_runway_end_type (String):
          The classification of the high end of the runway.

          * CONSTRUCTION_OR_ALTERATION_ON_AN_AIRPORT_WITH_LONGEST_RUNWAY_MORE_TH
          AN_3200_FEET-Construction on or alteration to a runway longer than
          3,200 feet with an imaginary surface that extends outward 20,000 feet
          and has a slope that does not exceed 100 to 1.

          * CONSTRUCTION_OR_ALTERATION_ON_AN_AIRPORT_WITH_LONGEST_RUNWAY_LESS_TH
          AN_3200_FEET-Construction on or alteration to a runway less than 3,200
          feet long with an imaginary surface that extends outward 10,000 feet
          and has a slope that does not exceed 50 to 1.

          * CONSTRUCTION_OR_ALTERATION_ON_A_HELIPORT-Construction on or
          alteration to a heliport landing and takeoff area with an imaginary
          surface that extends outward 5,000 feet and has a slope that does not
          exceed 25 to 1.

          * MILITARY_AIRPORT-Military airport runways are operated by an armed
          force of the United States. Primary surfaces are the same length as
          the runway. Primary surface width is 2,000 feet. Clear zone surface
          length is 1,000 feet, and width is the same as the primary surface.
          The approach clearance surface starts 200 feet beyond each end of the
          primary surface and extends for 50,000 feet. Approach surface width
          matches the primary surface width at the runway end but flares to a
          width of 16,000 feet at an elevation of 50,000 feet. Approach
          clearance surface slope is 50 to I to an elevation of 500 feet above
          airport elevation. It then rises horizontally to 50,000 feet.
          Transitional surface slope is 7 to I outward and upward at right
          angles to the runway centerline. See section 77.28 in the FAR Part 77
          specification for more information.

          *
          NONPRECISION_INSTRUMENT_RUNWAY_GREATER_THAN_(>)_3/4_MILE_VISIBILITY-A
          runway with a nonprecision instrument approach procedure that allows
          for landing in visibility conditions greater than three-quarters of a
          mile.

          * NONPRECISION_INSTRUMENT_RUNWAY_LESS_THAN_(<)_3/4_MILE_VISIBILITY-A
          runway with a nonprecision instrument approach procedure that allows
          for landing in visibility conditions less than three-quarters of a
          mile.

          * PRECISION_INSTRUMENT_RUNWAY-A runway that uses Instrument Landing
          System (ILS) or Precision Approach Radar (PAR) for approach
          procedures.

          * UTILITY_RUNWAY_VISUAL_APPROACH-A runway built for propeller aircraft
          not exceeding 12,500 pounds gross weight. Aircraft using the runway
          employ visual approach procedures.

          * UTILITY_RUNWAY_NON_PRECISION_INSTRUMENT_APPROACH-A runway built for
          propeller aircraft not exceeding 12,500 pounds gross weight. The
          runway has an instrument approach procedure that uses air navigation
          facilities with horizontal guidance. It can also have area-type
          navigation equipment with approved nonprecision instrument approach
          procedures.

          * VISUAL_RUNWAY_VISUAL_APPROACH-A runway that supports only visual
          approach procedures.
      low_runway_end_type (String):
          The classification of the low end of the runway.

          * SAME_AS_HIGH_RUNWAY_END_CLASSIFICATION-No low runway end type.

          *
          NONPRECISION_INSTRUMENT_RUNWAY_GREATER_THAN_(>)_3/4_MILE_VISIBILITY-A
          runway with a nonprecision instrument approach procedure that allows
          for landing in visibility conditions greater than three-quarters of a
          mile.

          * NONPRECISION_INSTRUMENT_RUNWAY_LESS_THAN_(<)_3/4_MILE_VISIBILITY-A
          runway with a nonprecision instrument approach procedure that allows
          for landing in visibility conditions less than three-quarters of a
          mile.

          * PRECISION_INSTRUMENT_RUNWAY-A runway that uses Instrument Landing
          System (ILS) or Precision Approach Radar (PAR) for approach
          procedures.

          * UTILITY_RUNWAY_VISUAL_APPROACH-A runway built for propeller aircraft
          not exceeding 12,500 pounds gross weight. Aircraft using the runway
          employ visual approach procedures.

          * UTILITY_RUNWAY_NON_PRECISION_INSTRUMENT_APPROACH-A runway built for
          propeller aircraft not exceeding 12,500 pounds gross weight. The
          runway has an instrument approach procedure that uses air navigation
          facilities with horizontal guidance. It can also have area-type
          navigation equipment with approved nonprecision instrument approach
          procedures.

          * VISUAL_RUNWAY_VISUAL_APPROACH-A runway that supports only visual
          approach procedures.
      specially_prepared_hard_surface_runway {Boolean}:
          A specially prepared hard surface indicates that the primary surface
          extends 200 feet beyond each end of the runway.

          *  SPECIALLY_PREPARED_HARD_SURFACE_RUNWAY-The runway has a specially
          prepared hard surface. This is the default.

          *  NON_SPECIALLY_PREPARED_HARD_SURFACE_RUNWAY-The runway does not have
          a specially prepared hard surface.
      highend_clear_way_length {Double}:
          The length of the area at the high end of the runway. The unit of
          measurement is based on the input runway features.
      lowend_clear_way_length {Double}:
          The length of the area at the low end of the runway. The unit of
          measurement is based on the input runway features.
      airport_elevation {Double}:
          The highest point on any runway in an airport. The unit of measurement
          is in meters. The default is the highest point from the runway end.
      include_merged_surface {Boolean}:
          Indicates whether merged horizontal and conical surfaces are included
          in the OIS in addition to the regular surfaces.

          * INCLUDE_MERGED_SURFACE-Include merged surfaces in the OIS output.
          This is the default.

          * NOT_INCLUDE_MERGED_SURFACE-Do not include merged surfaces in the OIS
          output."""
    from arcpy.geoprocessing._base import gp, gp_fixargs
    from arcpy.arcobjects.arcobjectconversion import convertArcObjectToPythonObject
    try:
        retval = convertArcObjectToPythonObject(gp.FAAFAR77_aviation(*gp_fixargs((in_features, target, high_runway_end_type, low_runway_end_type, specially_prepared_hard_surface_runway, highend_clear_way_length, lowend_clear_way_length, airport_elevation, include_merged_surface), True)))
        return retval
    except Exception as e:
        raise e

@gptooldoc('ICAOAnnex14_aviation', None)
def ICAOAnnex14(in_features=None, target=None, runway_type=None, highend_clear_way_length=None, lowend_clear_way_length=None, airport_elevation=None, runway_direction=None, include_merged_surface=None):
    """ICAOAnnex14_aviation(in_features, target, runway_type, {highend_clear_way_length}, {lowend_clear_way_length}, {airport_elevation}, {runway_direction}, {include_merged_surface})

        Creates obstruction identification surfaces based on the ICAO Annex
        14 specification. These surfaces define areas of airspace around an
        aerodrome. Each area defines the limits to which an obstruction can
        project into an airspace. The type, function, and dimension of a
        surface differ by runway classification. This tool creates surfaces as
        polygon or multipatch features.

     INPUTS:
      in_features (Feature Layer):
          The input runway dataset. The feature class must be z-enabled and
          contain polylines.
      target (Feature Layer):
          The output feature class that will contain the generated obstruction
          identification surfaces.
      runway_type (String):
          The runway classification of the in_features.

          * NON_INSTRUMENT_CODE_NUMBER_1-A runway intended for the operation of
          aircraft using visual approach procedures. Runway strip length is 30
          meters.

          * NON_INSTRUMENT_CODE_NUMBER_2-A runway with a 60-meter strip length
          and 40-meter strip width that is intended for the operation of
          aircraft using visual approach procedures.

          * NON_INSTRUMENT_CODE_NUMBER_3-A runway with a 60-meter strip length
          and 75-meter strip width that is intended for the operation of
          aircraft using visual approach procedures.

          * NON_INSTRUMENT_CODE_NUMBER_4-A runway with a 60-meter strip length
          and 75-meter strip width that is intended for the operation of
          aircraft using visual approach procedures.

          * NON_PRECISION_APPROACH_CODE_NUMBER_1-An instrument runway served by
          visual aids and a nonvisual aid providing at least directional
          guidance adequate for a straight-in approach. This runway type has a
          60-meter strip length and a 75-meter strip width on either side of the
          runway centre line.

          * NON_PRECISION_APPROACH_CODE_NUMBER_2-An instrument runway served by
          visual aids and a nonvisual aid providing at least directional
          guidance adequate for a straight-in approach. This runway type has a
          60-meter strip length and a 75-meter strip width on either side of the
          runway centre line.

          * NON_PRECISION_APPROACH_CODE_NUMBER_3-An instrument runway served by
          visual aids and a nonvisual aid providing at least directional
          guidance adequate for a straight-in approach. This runway type has a
          60-meter strip length and a 150-meter strip width on either side of
          the runway centre line.

          * NON_PRECISION_APPROACH_CODE_NUMBER_4-An instrument runway served by
          visual aids and a nonvisual aid providing at least directional
          guidance adequate for a straight-in approach. This runway type has a
          60-meter strip length and a 150-meter strip width on either side of
          the runway centre line.

          * PRECISION_APPROACH_CATEGORY_I_CODE_NUMBER_1-An instrument runway
          served by an Instrument Landing System (ILS) or a Microwave Landing
          System (MLS) and visual aids intended for operations with a decision
          height not lower than 60 meters (200 ft.) and either a visibility not
          less than 800 meters or a runway visual range not less than 550
          meters. This runway type has a 60-meter strip length and a 75-meter
          strip width on either side of the runway centre line.

          * PRECISION_APPROACH_CATEGORY_I_CODE_NUMBER_2-An instrument runway
          served by ILS and/or MLS and visual aids intended for operations with
          a decision height not lower than 60 meters (200 ft.) and either a
          visibility not less than 800 meters or a runway visual range not less
          than 550 meters. This runway type has a 60-meter strip length and a
          75-meter strip width on either side of the runway centre line.

          * PRECISION_APPROACH_CATEGORY_I_CODE_NUMBER_3_4-An instrument runway
          served by ILS and/or MLS and visual aids intended for operations with
          a decision height not lower than 60 meters (200 ft.) and either a
          visibility not less than 800 meters or a runway visual range not less
          than 550 meters. This runway type has a 60-meter strip length and a
          150-meter strip width on either side of the runway centre line.

          * PRECISION_APPROACH_CATEGORY_II_III_CODE NUMBER_3_4-An instrument
          runway served by ILS and/or MLS and visual aids intended for
          operations with a decision height lower than 60 meters (200 ft.) but
          not lower than 30 meters (100 ft.) and a runway visual range not less
          than 350 meters. This runway type has a 60-meter strip length and a
          150-meter strip width on either side of the runway centre line.
      highend_clear_way_length {Double}:
          The length of the area at the high end of the runway. The unit of
          measurement is based on the input runway features.
      lowend_clear_way_length {Double}:
          The length of the area at the low end of the runway. The unit of
          measurement is based on the input runway features.
      airport_elevation {Double}:
          The highest point on any runway in an airport. The unit of measurement
          is in meters. The default is the highest point from the runway end.
      runway_direction {String}:
          The direction of the in_features.

          * HIGH_END_TO_LOW_END-From runway high end to low end

          * LOW_END_TO_HIGH_END-From runway low end to high end
      include_merged_surface {Boolean}:
          Indicates whether merged surfaces are generated.

          * INCLUDE_MERGED_SURFACE-All the surfaces are generated for the merged
          surfaces, as well as merged conical and horizontal surfaces. This is
          the default.

          * NOT_INCLUDE_MERGED_SURFACE-Surfaces are not generated for the merged
          surfaces."""
    from arcpy.geoprocessing._base import gp, gp_fixargs
    from arcpy.arcobjects.arcobjectconversion import convertArcObjectToPythonObject
    try:
        retval = convertArcObjectToPythonObject(gp.ICAOAnnex14_aviation(*gp_fixargs((in_features, target, runway_type, highend_clear_way_length, lowend_clear_way_length, airport_elevation, runway_direction, include_merged_surface), True)))
        return retval
    except Exception as e:
        raise e

@gptooldoc('ICAOAnnex15_aviation', None)
def ICAOAnnex15(in_features=None, target=None, highend_clear_way_length=None, lowend_clear_way_length=None):
    """ICAOAnnex15_aviation(in_features, target, {highend_clear_way_length}, {lowend_clear_way_length})

        Creates obstruction identification surfaces based on the ICAO Annex
        15 specification. These surfaces define areas of airspace around an
        aerodrome. Each area defines the limits to which an obstruction can
        project into an airspace. This tool creates surfaces as polygon or
        multipatch features.

     INPUTS:
      in_features (Feature Layer):
          The input runway dataset. The feature class must be z-enabled and
          contain polylines.
      target (Feature Layer):
          The target feature class that will contain the generated obstruction
          identification surfaces.
      highend_clear_way_length {Double}:
          The length of the area at the high end of the runway. The unit of
          measurement of based on the input runway features.
      lowend_clear_way_length {Double}:
          The length of the area at the low end of the runway. The unit of
          measurement is based on the input runway features."""
    from arcpy.geoprocessing._base import gp, gp_fixargs
    from arcpy.arcobjects.arcobjectconversion import convertArcObjectToPythonObject
    try:
        retval = convertArcObjectToPythonObject(gp.ICAOAnnex15_aviation(*gp_fixargs((in_features, target, highend_clear_way_length, lowend_clear_way_length), True)))
        return retval
    except Exception as e:
        raise e

@gptooldoc('UnifiedFacilitiesCriteria_aviation', None)
def UnifiedFacilitiesCriteria(in_runway_features=None, target_ois_features=None, in_wing_type=None, in_service_type=None, in_runway_class=None, in_flight_rule=None, highend_clear_way_length=None, lowend_clear_way_length=None, airport_elevation=None):
    """UnifiedFacilitiesCriteria_aviation(in_runway_features, target_ois_features, in_wing_type, in_service_type, in_runway_class, in_flight_rule, {highend_clear_way_length}, {lowend_clear_way_length}, {airport_elevation})

        Creates Unified Facilities Criteria (UFC) surfaces. Generates a
        primary surface, an approach surface, an inner horizontal surface, a
        conical surface, a transitional surface, a clear zone surface, an
        accident potential zone surface, and an outer horizontal surface.

     INPUTS:
      in_runway_features (Feature Layer):
          The input runway dataset. The feature class must be z-enabled and
          contain polylines.
      target_ois_features (Feature Layer):
          The existing output feature class that will contain the generated UFC
          surfaces.
      in_wing_type (String):
          The type of the aircraft.

          * FIXED-Fixed wing type.

          * ROTARY-Rotary wing type.
          If you choose Rotary, the Runway Class parameter will default to Class
          A without having any effect on the surface generation.
      in_service_type (String):
          The type of military service.

          * AIRFORCE-Air Force service type.

          * ARMY-Army service type.

          * NAVY-Navy service type.

          * MARINECORPS-Marine Corps service type.
      in_runway_class (String):
          Runways are classified as either Class A or Class B based on aircraft
          type.

          * CLASS_A-Class A runway classification.

          * CLASS_B-Class B runway classification.
      in_flight_rule (String):
          Rules that govern the procedures for conducting flight, either
          instrument or under visual conditions.

          * INSTRUMENT-Instrument flight condition.

          * VISUAL-Visual flight condition.
      highend_clear_way_length {Double}:
          The length of the area at the high end of the runway. The unit of
          measurement is based on the input runway features.
      lowend_clear_way_length {Double}:
          The length of the area at the low end of the runway. The unit of
          measurement is based on the input runway features.
      airport_elevation {Double}:
          The highest point on any runway in an airport. The unit of measurement
          is meters. The default is the highest point from the runway end."""
    from arcpy.geoprocessing._base import gp, gp_fixargs
    from arcpy.arcobjects.arcobjectconversion import convertArcObjectToPythonObject
    try:
        retval = convertArcObjectToPythonObject(gp.UnifiedFacilitiesCriteria_aviation(*gp_fixargs((in_runway_features, target_ois_features, in_wing_type, in_service_type, in_runway_class, in_flight_rule, highend_clear_way_length, lowend_clear_way_length, airport_elevation), True)))
        return retval
    except Exception as e:
        raise e


# End of generated toolbox code
del gptooldoc, gp, gp_fixargs, convertArcObjectToPythonObject