# -*- coding: utf-8 -*-
#COPYRIGHT 2018 ESRI
#
#TRADE SECRETS: ESRI PROPRIETARY AND CONFIDENTIAL
#Unpublished material - all rights reserved under the
#Copyright Laws of the United States.
#
#For additional information, contact:
#Environmental Systems Research Institute, Inc.
#Attn: Contracts Dept
#380 New York Street
#Redlands, California, USA 92373
#
#email: contracts@esri.com
r"""Geocoding is the process of assigning a location, usually in the form
of coordinate values, to an address by comparing the descriptive
location elements in the address to those present in the reference
material. Addresses come in many forms, ranging from the common
address format of house number followed by the street name and
succeeding information to other location descriptions such as postal
zone or census tract. In essence, an address includes any type of
information that distinguishes a place."""
__all__ = ['ConsolidateLocator', 'CreateAddressLocator',
           'CreateCompositeAddressLocator', 'GeocodeAddresses',
           'GeocodeLocationsFromTable', 'PackageLocator',
           'RebuildAddressLocator', 'RematchAddresses', 'ReverseGeocode',
           'StandardizeAddresses']
__alias__ = 'geocoding'
from arcpy.geoprocessing._base import gptooldoc, gp, gp_fixargs
from arcpy.arcobjects.arcobjectconversion import convertArcObjectToPythonObject

# Tools
@gptooldoc('ConsolidateLocator_geocoding', None)
def ConsolidateLocator(in_locator=None, output_folder=None, copy_arcsde_locator=None):
    """ConsolidateLocator_geocoding(in_locator, output_folder, {copy_arcsde_locator})

        Consolidate a locator or composite locator by copying all locators
        into a single folder.

     INPUTS:
      in_locator (Address Locator):
          The input locator or composite locator that will be consolidated.
      copy_arcsde_locator {Boolean}:
          This parameter has no effect in ArcGIS Pro. It remains only to
          support backward compatibility.

     OUTPUTS:
      output_folder (Folder):
          The output folder that will contain the consolidated locator or
          composite locator with its participating locators."""
    from arcpy.geoprocessing._base import gp, gp_fixargs
    from arcpy.arcobjects.arcobjectconversion import convertArcObjectToPythonObject
    try:
        retval = convertArcObjectToPythonObject(gp.ConsolidateLocator_geocoding(*gp_fixargs((in_locator, output_folder, copy_arcsde_locator), True)))
        return retval
    except Exception as e:
        raise e

@gptooldoc('CreateAddressLocator_geocoding', None)
def CreateAddressLocator(in_address_locator_style=None, in_reference_data=None, in_field_map=None, out_address_locator=None, config_keyword=None, enable_suggestions=None):
    """CreateAddressLocator_geocoding(in_address_locator_style, in_reference_data;in_reference_data..., in_field_map, out_address_locator, {config_keyword}, {enable_suggestions})

        Creates an address locator. The address locator can be used to find a
        location of an address, geocode a table of addresses, or get the
        address of a point location.

     INPUTS:
      in_address_locator_style (Address Locator Style):
          The address locator style on which to base the new address locator.

          * US Address-Dual Ranges-Use when the reference data is a line feature
          class that contains house number ranges on both sides of a street
          segment, and you want to search for an address on a specific side of
          the street or a street intersection.

          * US Address-One Range-Use when the reference data is a line feature
          class with one house number range for each road segment, and you want
          to search for an address (where side is not needed) or a street
          intersection.

          *  US Address-Single House-Use when the reference data is a point or
          polygon feature class where each feature represents a single address,
          and you want to search for an exact address for a parcel, building, or
          address point.

          *  US Address-Single House Subaddress-Use when the reference data is a
          point or polygon feature class where each feature represents a single
          address with optional subaddress elements, and you want to search for
          apartment units, townhouses, duplexes, or stores in a shopping plaza.

          *  US Address-Street Name-Use when the reference data is a line
          feature class, and you want to search for addresses using only the
          street name. Address range information is not required.

          *  US Address-City State-Use when the reference data is a point or
          polygon feature class, and you want to search for a specific city in a
          state.

          *  US Address-ZIP 5 Digit-Use when the reference data is a point or
          polygon feature class where each feature represents a ZIP Code
          centroid or geographic area, and you want to search for a specific ZIP
          Code location.

          *  General-Gazetteer-Use when the reference data is a point or polygon
          feature class where each feature represents any unique feature, and
          you want to search for a place-name, landmark, water meter, or any
          unique value.
      in_reference_data (Value Table):
          The reference data feature classes and tables, along with their roles,
          to be used by the address locator.

          * Primary table-Defines the primary reference dataset feature class
          for a locator, such as a street centerline feature class. This is a
          required table.

          * Alternate City Name table-Defines an alternate city name table that
          contains alternate names for the city or place-name. The table is
          required to have a JoinID that can be used to join to the primary
          table. This table is optional

          * Alternate Name table-Defines an alternate street name table that
          contains alternate names for the street or point features. The table
          is required to have a JoinID that can be used to join to the primary
          table. This table is optional.

          * Alias table-Defines a place-name alias table that contains place-
          names and actual addresses for the names. Users can find the location
          using either the place-name such as Field Museum or the address 1400 S
          Lakeshore Drive Chicago, IL 60605. This table is optional.
      in_field_map (Field Info):
          The mapping of reference data fields used by the address locator style
          to fields in the reference datasets. Each field mapping in this
          parameter is in the following format:   # <locator field alias>
          <dataset field name> # This shows as an
          example: reference_data_field_map = ''' "'Feature ID' FeatureID;'*From
          Left' L_F_ADD; '*To Left' L_T_ADD;'*From Right' R_F_ADD; '*To Right'
          R_T_ADD;'Prefix Direction' PREFIX; 'Prefix Type' PRE_TYPE;'*Street
          Name' NAME; 'Suffix Type' TYPE;'Suffix Direction' SUFFIX; 'Left City
          or Place' CITYL;'Right City or Place' CITYR;'Left ZIP Code'
          ZIPL;'Right ZIP Code' ZIPR;'Left State' State_Abbr;'Right State'
          State_Abbr" '''where <locator field alias> is the alias name for the
          reference data
          field used by the address locator, and <dataset field name> is the
          name of the field in the reference dataset. Fields with an asterisk
          (*) next to their names are required by the address locator style.If
          you choose not to map an optional reference data field used by the
          address locator style to a field in a reference dataset, specify that
          there is no mapping by using <None> in place of a field name.To
          determine the alias name for a reference data field used by a
          locator style, open the Create Address Locator tool and choose the
          locator style. The name that appears in the Field Name column of the
          Field Map parameter is the field's alias name.
      config_keyword {String}:
          This parameter has no effect in ArcGIS Pro. It remains to support
          backward compatibility.
      enable_suggestions {Boolean}:
          Allows character-by-character autocomplete suggestions to be generated
          for user input in a client application. This capability facilitates
          the interactive search user experience by reducing the number of
          characters that need to be typed before a suggested match is obtained.
          The idea is that a client application can provide a list of
          suggestions that is updated with each character entered by a user
          until the place they are looking for is returned in the list.Only the
          locator styles provided by Esri can be used to build locators
          with suggestions enabled. The suggestion functionality is only exposed
          when such a locator is published as a geocode service.

          * ENABLED-Suggestions are enabled for the locator.

          * DISABLED-Suggestions are not enabled for the locator. This is the
          default.

     OUTPUTS:
      out_address_locator (Address Locator):
          The locator to create in a file folder. Saving the locator to a file
          folder allows you to take advantage of performance improvements,
          multithreading capabilities, and suggestions support. Once the locator
          is created, additional properties and options can be modified in the
          locator's settings."""
    from arcpy.geoprocessing._base import gp, gp_fixargs
    from arcpy.arcobjects.arcobjectconversion import convertArcObjectToPythonObject
    try:
        retval = convertArcObjectToPythonObject(gp.CreateAddressLocator_geocoding(*gp_fixargs((in_address_locator_style, in_reference_data, in_field_map, out_address_locator, config_keyword, enable_suggestions), True)))
        return retval
    except Exception as e:
        raise e

@gptooldoc('CreateCompositeAddressLocator_geocoding', None)
def CreateCompositeAddressLocator(in_address_locators=None, in_field_map=None, in_selection_criteria=None, out_composite_address_locator=None):
    """CreateCompositeAddressLocator_geocoding(in_address_locators;in_address_locators..., in_field_map, {in_selection_criteria;in_selection_criteria...}, out_composite_address_locator)

        Creates a composite address locator. A composite locator consists of
        two or more individual locators that allow addresses to be matched
        against the multiple locators.

     INPUTS:
      in_address_locators (Value Table):
          The order of the participating address locators determines how
          candidates are searched and an address is matched. When you geocode a
          single address, the address will be matched against all participating
          address locators unless the locator is specified with a selection
          criterion. All the found candidates will be displayed based on the
          order of the listed participating address locators. If you geocode a
          table of addresses, addresses are matched automatically to the first
          best candidate found from the first participating address locators. If
          the address fails to match, it will fall back to the subsequent
          locator in the list.A reference name for each participating locator is
          required. This is
          the name of the locator referred to by the composite locator. The name
          should contain no space or special symbols. The maximum length of the
          name is 14 characters.
      in_field_map (Field Mappings):
          The mapping of input fields used by each participating locator to the
          input fields of the composite address locator.
      in_selection_criteria {Value Table}:
          Selection criteria for each participating locator. Only one selection
          criterion is supported for each participating address locator. Using
          selection criteria will disqualify participating address
          locators that do not meet the criteria on a particular address so that
          the geocoding process will be more efficient. Refer to the topic
          Create a composite locator to learn more about the use of selection
          criteria in the geocoding process.

     OUTPUTS:
      out_composite_address_locator (Address Locator):
          The composite address locator to create. ArcGIS Pro only supports
          saving locators in a file folder."""
    from arcpy.geoprocessing._base import gp, gp_fixargs
    from arcpy.arcobjects.arcobjectconversion import convertArcObjectToPythonObject
    try:
        retval = convertArcObjectToPythonObject(gp.CreateCompositeAddressLocator_geocoding(*gp_fixargs((in_address_locators, in_field_map, in_selection_criteria, out_composite_address_locator), True)))
        return retval
    except Exception as e:
        raise e

@gptooldoc('GeocodeAddresses_geocoding', None)
def GeocodeAddresses(in_table=None, address_locator=None, in_address_fields=None, out_feature_class=None, out_relationship_type=None, country=None, location_type=None, category=None):
    """GeocodeAddresses_geocoding(in_table, address_locator, in_address_fields, out_feature_class, {out_relationship_type}, {country;country...}, {location_type}, {category;category...})

        Geocodes a table of addresses. This process requires a table that
        stores the addresses you want to geocode and an address locator or a
        composite address locator. This tool matches the addresses against the
        locator and saves the result for each input record in a new point
        feature class. When using the ArcGIS World Geocoding Service, this
        operation may consume credits.

     INPUTS:
      in_table (Table View):
          The table of addresses to geocode.
      address_locator (Address Locator):
          The address locator to use to geocode the table of addresses.Including
          the .loc extension after the locator name at the end of the
          locator path is optional.
      in_address_fields (Field Info):
          Each field mapping in this parameter is in the format
          input_address_field, table_field_name where input_address_field is the
          name of the input address field specified by the address locator, and
          table_field_name is the name of the corresponding field in the table
          of addresses you want to geocode.You can specify one single input
          field that stores the complete
          address.  Alternatively, you can also specify multiple fields if the
          input addresses are split into multiple fields such as Address, City,
          State, and ZIP for a general United States address.If you choose not
          to map an optional input address field used by the
          address locator to a field in the input table of addresses, specify
          that there is no mapping by using <None> in place of a field name.
      out_relationship_type {Boolean}:
          This parameter has no effect in ArcGIS Pro. It remains to support
          backward compatibility with ArcGIS Desktop. In ArcGIS Pro, the only
          permissible value is STATIC.

          * STATIC-Creates a static copy of the fields input address table in
          the output feature class. This is the only permissible value.

          * DYNAMIC-This option is not applicable in ArcGIS Pro. See ArcGIS
          Desktop help for this tool.
      country {String}:
          This parameter is available for locators that support a country
          parameter and will limit geocoding to the selected countries. Making a
          country selection will improve the accuracy of geocoding in most
          cases. If a field representing countries in the Input Table is mapped
          to the Country Input Address Field, the country value from the Input
          Table will override the Country parameter.This is limited to the
          selected country or countries, and when no
          country is specified, geocoding is performed against all supported
          countries of the locator.Country is not supported for all
          locators.Input should be specified as ISO 3166-1 3-character country
          codes.
      location_type {String}:
          Specifies the preferred output geometry for POINT_ADDRESS matches. The
          options for this parameter are ROUTING_LOCATION, the side of street
          location, which can be used for routing, or ADDRESS_LOCATION, the
          location that represents the rooftop, parcel centroid for the address,
          or front door. If the preferred location does not exist in the data,
          the default location of ROUTING_LOCATION will be returned instead. For
          geocode results with Addr_type=PointAddress, the x,y attribute values
          describe the coordinates of the address along the street, while the
          DisplayX and DisplayY values describe the rooftop or building centroid
          coordinates.This parameter is not supported for all locators.

          * ADDRESS_LOCATION-Returns geometry for geocode results that represent
          an address location such as rooftop location, parcel centroid, or
          front door.

          * ROUTING_LOCATION-Returns geometry for geocode results that represent
          a location close to the side of the street, which can be used for
          vehicle routing. This is the default.
      category {String}:
          Limits the types of places the locator searches, thus eliminating
          false positive matches and potentially speeding up the search process.
          When no category is used, geocoding is performed against all supported
          categories. Not all category values are supported for all locations
          and countries. In general, the parameter can be used for the
          following:

          * Limit matches to specific place types or address levels

          * Avoid fallback matches to unwanted address levels

          * Disambiguate coordinate searches
          This parameter is not supported for all locators.

     OUTPUTS:
      out_feature_class (Feature Class):
          The output geocoded feature class or shapefile."""
    from arcpy.geoprocessing._base import gp, gp_fixargs
    from arcpy.arcobjects.arcobjectconversion import convertArcObjectToPythonObject
    try:
        retval = convertArcObjectToPythonObject(gp.GeocodeAddresses_geocoding(*gp_fixargs((in_table, address_locator, in_address_fields, out_feature_class, out_relationship_type, country, location_type, category), True)))
        return retval
    except Exception as e:
        raise e

@gptooldoc('PackageLocator_geocoding', None)
def PackageLocator(in_locator=None, output_file=None, copy_arcsde_locator=None, additional_files=None, summary=None, tags=None):
    """PackageLocator_geocoding(in_locator, output_file, {copy_arcsde_locator}, {additional_files;additional_files...}, {summary}, {tags})

        Package a locator or composite locator to create a single compressed
        .gcpk file.

     INPUTS:
      in_locator (Address Locator):
          The locator or composite locator that will be packaged.
      copy_arcsde_locator {Boolean}:
          This parameter has no effect in ArcGIS Pro. It remains only to
          support backward compatibility.
      additional_files {File}:
          Adds additional files to a package. Additional files, such as .doc,
          .txt, .pdf, and so on, are used to provide more information about the
          contents and purpose of the package.
      summary {String}:
          Adds summary information to the properties of the package.
      tags {String}:
          Adds tag information to the properties of the package. Multiple tags
          can be added or separated by a comma or semicolon.

     OUTPUTS:
      output_file (File):
          The name and location of the output locator package (.gcpk)."""
    from arcpy.geoprocessing._base import gp, gp_fixargs
    from arcpy.arcobjects.arcobjectconversion import convertArcObjectToPythonObject
    try:
        retval = convertArcObjectToPythonObject(gp.PackageLocator_geocoding(*gp_fixargs((in_locator, output_file, copy_arcsde_locator, additional_files, summary, tags), True)))
        return retval
    except Exception as e:
        raise e

@gptooldoc('RebuildAddressLocator_geocoding', None)
def RebuildAddressLocator(in_address_locator=None):
    """RebuildAddressLocator_geocoding(in_address_locator)

        Rebuilds an address locator to update the locator with the current
        reference data. Since an address locator contains a snapshot of the
        reference data when it was created, it will not geocode addresses
        against the updated data when the geometry and attributes of the
        reference data are changed. To geocode addresses against the current
        version of the reference data, the address locator must be rebuilt if
        you want to update the changes in the locator.

     INPUTS:
      in_address_locator (Address Locator):
          The address locator to rebuild."""
    from arcpy.geoprocessing._base import gp, gp_fixargs
    from arcpy.arcobjects.arcobjectconversion import convertArcObjectToPythonObject
    try:
        retval = convertArcObjectToPythonObject(gp.RebuildAddressLocator_geocoding(*gp_fixargs((in_address_locator,), True)))
        return retval
    except Exception as e:
        raise e

@gptooldoc('RematchAddresses_geocoding', None)
def RematchAddresses(in_geocoded_feature_class=None, in_where_clause=None):
    """RematchAddresses_geocoding(in_geocoded_feature_class, {in_where_clause})

        Rematches addresses in a geocoded feature class.

     INPUTS:
      in_geocoded_feature_class (Feature Class):
          The geocoded feature class you want to rematch.
      in_where_clause {SQL Expression}:
          An SQL expression used to select a subset of features. For more
          information on SQL syntax see the help topic SQL reference for
          elements used in query expressions."""
    from arcpy.geoprocessing._base import gp, gp_fixargs
    from arcpy.arcobjects.arcobjectconversion import convertArcObjectToPythonObject
    try:
        retval = convertArcObjectToPythonObject(gp.RematchAddresses_geocoding(*gp_fixargs((in_geocoded_feature_class, in_where_clause), True)))
        return retval
    except Exception as e:
        raise e

@gptooldoc('ReverseGeocode_geocoding', None)
def ReverseGeocode(in_features=None, in_address_locator=None, out_feature_class=None, address_type=None, search_distance=None, feature_type=None, location_type=None):
    """ReverseGeocode_geocoding(in_features, in_address_locator, out_feature_class, {address_type}, search_distance, {feature_type;feature_type...}, {location_type})

        Creates addresses from point locations in a feature class. The
        reverse geocoding process searches for the nearest address or
        intersection for the point location based on the specified search
        distance. When using the ArcGIS World Geocoding Service, this
        operation may consume credits.

     INPUTS:
      in_features (Feature Layer):
          A point feature class or layer from which addresses are returned based
          on the features' point location.
      in_address_locator (Address Locator):
          The address locator to use to reverse geocode the input feature
          class.
      address_type {String}:
          Indicates whether to return addresses for the points as street
          addresses or intersection addresses if the address locator supports
          intersection matching.

          * ADDRESS-Returns street addresses or in the format defined by the
          input address locator. This is the default option.

          * INTERSECTION-Returns intersection addresses. This option is
          available if the address locator supports matching intersection
          addresses.
      search_distance (Linear Unit):
          The distance used to search for the nearest address or intersection
          for the point location. Some locators use optimized distance values
          that do not support overriding the search distance parameter.
      feature_type {String}:
          Limits the possible match types returned. Single or multiple values
          can be selected. If a single value is selected, the search tolerance
          for the input feature type is 500 meters. If multiple values are
          included, the default search distances are applied.This parameter is
          not supported for all locators.

          * POINT_ADDRESS-A street address based on points that represent house
          and building locations.

          * STREET_ADDRESS-A street address that differs from Point Address
          because the house number is interpolated from a range of numbers.
          Street Address matches include the house number range for the matching
          street segment rather than the interpolated house number value.

          * STREET_INTERSECTION-A street address consisting of a street
          intersection along with city and optional state and postal code
          information. This is derived from Street Address reference data, for
          example, Redlands Blvd & New York St, Redlands, CA, 92373.

          * LOCALITY-A place-name representing a populated place.

          * POSTAL-A postal code. Reference data is postal code points, for
          example, 90210 USA.

          * POINT_OF_INTEREST-A point of interest. Reference data consists of
          administrative division place-names, businesses, landmarks, and
          geographic features, for example, Starbucks.
      location_type {String}:
          Specifies the preferred output geometry for POINT_ADDRESS matches. The
          options for this parameter are ROUTING_LOCATION, the side of street
          location, which can be used for routing, or ADDRESS_LOCATION, the
          location that represents the rooftop, parcel centroid for the address,
          or front door. If the preferred location does not exist in the data,
          the default location of ROUTING_LOCATION will be returned instead. For
          geocode results with Addr_type=PointAddress, the x,y attribute values
          describe the coordinates of the address along the street, while the
          DisplayX and DisplayY values describe the rooftop or building centroid
          coordinates.This parameter is not supported for all locators.

          * ADDRESS_LOCATION-Returns geometry for geocode results that represent
          an address location such as rooftop, building centroid, or front door.

          * ROUTING_LOCATION-Returns geometry for geocode results that represent
          a location close to the side of the street, which can be used for
          vehicle routing. This is the default.

     OUTPUTS:
      out_feature_class (Feature Class):
          The output feature class."""
    from arcpy.geoprocessing._base import gp, gp_fixargs
    from arcpy.arcobjects.arcobjectconversion import convertArcObjectToPythonObject
    try:
        retval = convertArcObjectToPythonObject(gp.ReverseGeocode_geocoding(*gp_fixargs((in_features, in_address_locator, out_feature_class, address_type, search_distance, feature_type, location_type), True)))
        return retval
    except Exception as e:
        raise e

@gptooldoc('StandardizeAddresses_geocoding', None)
def StandardizeAddresses(in_address_data=None, in_input_address_fields=None, in_address_locator_style=None, in_output_address_fields=None, out_address_data=None, in_relationship_type=None):
    """StandardizeAddresses_geocoding(in_address_data, in_input_address_fields;in_input_address_fields..., in_address_locator_style, in_output_address_fields;in_output_address_fields..., out_address_data, {in_relationship_type})

        Standardizes the address information in a table or feature class.
        Standardizing the address components in your reference data into
        multiple address fields was a necessary step before building an
        address locator prior to ArcGIS Desktop 10.0. This step is no longer
        required because the standardization process occurs when the address
        locator is built regardless of whether the address components are
        stored in a single field or split across multiple fields.
        Standardizing the reference data and normalizing abbreviations and
        periods is not recommended at ArcGIS Desktop 10.0 and later because
        they will not improve geocoding performance or quality and they are
        now done during the geocoding process. Addresses are often presented
        in different forms that may contain
        various abbreviations of words, such as W for WEST or ST for STREET.
        Based on an address style you select,  the address can be split into
        multiple parts, such as House Number, Prefix Direction, Prefix Type,
        Street Name Suffix Type, Unit Type, and Unit Number. The address style
        specifies the components of an address and determines how the
        components are ordered and standardized; however, there is no
        guarantee how the data will be standardized based on the logic in the
        address locator style and the input data. Depending on the
        application, some address styles may expand the value of a word
        instead of abbreviating it.The input address you want to standardize
        can be stored in a single
        field. If the address information has already been split into multiple
        fields in the input feature class or table, this tool can concatenate
        the fields on the fly and standardize the information.

     INPUTS:
      in_address_data (Table View):
          The table or feature class containing address information that you
          want to standardize.
      in_input_address_fields (String):
          The set of fields in the input table or feature class that, when
          concatenated, forms the address to be standardized.
      in_address_locator_style (Address Locator Style):
          The address locator style to use to standardize the address
          information in the input table or feature class.
      in_output_address_fields (String):
          The set of standardized address fields to include in the output table
          or feature class.
      in_relationship_type {Boolean}:
          Indicates whether to create a static or dynamic output dataset.

          * Static-Creates an output table or feature class that contains a copy
          of the rows or features in the input table and the standardized
          address fields. This is the default.

          * Dynamic-Creates a table containing the standardized address fields
          and a relationship class that joins to the input table or feature
          class. The option is only supported if both the input and output
          datasets are
          stored in the same geodatabase workspace.

     OUTPUTS:
      out_address_data (Dataset):
          The output table or feature class to create containing the
          standardized address fields."""
    from arcpy.geoprocessing._base import gp, gp_fixargs
    from arcpy.arcobjects.arcobjectconversion import convertArcObjectToPythonObject
    try:
        retval = convertArcObjectToPythonObject(gp.StandardizeAddresses_geocoding(*gp_fixargs((in_address_data, in_input_address_fields, in_address_locator_style, in_output_address_fields, out_address_data, in_relationship_type), True)))
        return retval
    except Exception as e:
        raise e


# Portal toolset
@gptooldoc('GeocodeLocationsFromTable_geocoding', None)
def GeocodeLocationsFromTable(in_table=None, in_address_locator=None, address_fields=None, output_name=None, country=None, location_type=None, category=None):
    """GeocodeLocationsFromTable_geocoding(in_table, in_address_locator, address_fields, output_name, {country;country...}, {location_type}, {category;category...})

        Converts large hosted tables of addresses or places into points.This
        tool geocodes hosted tables using locators hosted on an ArcGIS
        Enterprise portal, which creates a hosted feature layer containing the
        geocoded results. This tool is designed to automate the process of
        geocoding tables published to the portal and saving the output to the
        portal. If you are not interested in saving the geocode results to the
        portal, use the Geocode Table pane or the Geocode Addresses tool.

     INPUTS:
      in_table (Record Set):
          The table on the portal that contains addresses or places to geocode.
      in_address_locator (Address Locator):
          The portal locator that will be used to geocode the input table from
          the portal.
      address_fields (Field Info):
          Each field mapping in this parameter is in the format
          input_locator_field, table_field_name where input_locator_field is the
          name of the input address field specified by the locator, and
          table_field_name is the name of the corresponding field in the table
          of addresses you want to geocode.You can specify one single input
          field that stores the complete
          address.  Alternatively, you can specify multiple fields if the input
          addresses are split into multiple fields such as Address, City, State,
          and ZIP for a general United States address.If you choose not to map
          an optional input address field used by the
          locator to a field in the input table of addresses, specify that there
          is no mapping by using <None> in place of a field name.
      output_name (String):
          The name of the output geocoded feature layer that will be created on
          the portal.
      country {String}:
          The country or countries to search for the geocoded addresses. This
          parameter is available for locators that support a country parameter
          and will limit geocoding to the selected countries. Making a country
          selection will improve the accuracy of geocoding in most cases. If a
          field representing countries in the Input Table is mapped to the
          Country Input Address Field, the country value from the Input Table
          will override the Country parameter.If no country is specified,
          geocoding is performed against all
          supported countries of the locator.Country is not supported for all
          locators.Input should be specified as ISO 3166-1 3-character country
          codes.
      location_type {String}:
          Specifies the preferred output geometry for POINT_ADDRESS matches. The
          options for this parameter are ROUTING_LOCATION, the side of street
          location that can be used for routing, or ADDRESS_LOCATION, the
          location that represents the rooftop, parcel centroid for the address,
          or front door. If the preferred location does not exist in the data,
          the default location of ROUTING_LOCATION will be returned instead. For
          geocode results with Addr_type=PointAddress, the x,y attribute values
          describe the coordinates of the address along the street, while the
          DisplayX and DisplayY values describe the rooftop or building centroid
          coordinates.This parameter is not supported for all locators.

          * ADDRESS_LOCATION-Returns geometry for geocode results that represent
          an address location such as a rooftop location, parcel centroid, or
          front door.

          * ROUTING_LOCATION-Returns geometry for geocode results that represent
          a location close to the side of the street that can be used for
          vehicle routing. This is the default.
      category {String}:
          Limits the types of places the locator searches, thus eliminating
          false positive matches and potentially speeding up the search process.
          Not all category values are supported for all locations and countries.
          In general, the Category parameter can be used for the following:

          * Limit matches to specific place types or address levels

          * Avoid fallback matches to unwanted address levels

          * Disambiguate coordinate searches
          When no category is used, geocoding is performed against all supported
          categories.This parameter is not supported for all locators."""
    from arcpy.geoprocessing._base import gp, gp_fixargs
    from arcpy.arcobjects.arcobjectconversion import convertArcObjectToPythonObject
    try:
        retval = convertArcObjectToPythonObject(gp.GeocodeLocationsFromTable_geocoding(*gp_fixargs((in_table, in_address_locator, address_fields, output_name, country, location_type, category), True)))
        return retval
    except Exception as e:
        raise e


# End of generated toolbox code
del gptooldoc, gp, gp_fixargs, convertArcObjectToPythonObject