#COPYRIGHT 2018 ESRI
#
#TRADE SECRETS: ESRI PROPRIETARY AND CONFIDENTIAL
#Unpublished material - all rights reserved under the
#Copyright Laws of the United States.
#
#For additional information, contact:
#Environmental Systems Research Institute, Inc.
#Attn: Contracts Dept
#380 New York Street
#Redlands, California, USA 92373
#
#email: contracts@esri.com
from __future__ import print_function

import os.path
import types

import arcgisscripting
from arcpy import env, gp

from . import Utils



useLocalFunctions = True

def SetUseLocalFunctions(value):
  global useLocalFunctions
  useLocalFunctions=value

def tryLocalFunction(*args):
  """If it returns None, no local function implementation is avail,
     otherwise it returns a Raster object
  """
  assert isinstance(args, tuple)
  assert len(args) >= 1

  canUse = True # env.outputCoordinateSystem is None
  canUse = canUse and env.mask is None

  if useLocalFunctions and canUse:
    if arcgisscripting._hasLocalFunctionRasterImplementation(args[0]):
      # 0: the operation name
      # 1: a scratch name already reserved to use later in save
      flatList = [args[0], Utils.scratchName()]

      for arg in args[1:]:
        # Flatten lists. Argument can be repeatable, provided as a list.
        if isinstance(arg, list):
          flatList += arg
        else:
          flatList.append(arg)

      return Raster(flatList), True

  return None, False

class Raster(arcgisscripting.Raster):
  """Raster(in_raster)

  Create a Raster object.

  Arguments:
  in_raster -- Name of raster
  """
  __esri_toolinfo__ = [
    "RasterLayer|RasterDataset|RasterBand::::"
  ]

  @staticmethod
  def _datasetNameWithWorkspace(name):
    if not os.path.isabs(name):
      # If relative then this is the GP default behaviour for outputs.
      if env.workspace:
        name = os.path.join(env.workspace, name)
      elif env.scratchWorkspace:
        name = os.path.join(env.scratchWorkspace, name)
      elif os.environ.has_key("TEMP"):
        name = os.path.join(os.environ["TEMP"], name)
    return name



  def __init__(self,
               args):
    # args is list (LocalFunction) or catalogPath value
    arcgisscripting.Raster.__init__(self, args)

  def __del__(self):
    """ if not permanent then remove dataset"""
    # TODO should we do this in arcgisscripting.Raster
    if (not self.permanent) and len(self.catalogPath):
      if gp.exists(self.catalogPath): # TODO will gp accept Raster here?
        try:
          gp.delete_management(self.catalogPath)
        except:
          print("Warning: temporary dataset", self.catalogPath, "is not removed")

  def __nonzero__(self):
    """Evaluate raster to single boolean"""
    # This is what numpy raises:
    # ValueError: The truth value of an array with more than one element is
    #             ambiguous. Use a.any() or a.all()
    raise ValueError("The truth value of a raster is ambiguous. Use Con()")

  def __str__(self):
    if len(self.catalogPath):
       return str(self.catalogPath)
    return "<Raster Object>"

  def __repr__(self):
    return repr(self.catalogPath)

  def save(self, name):
    """Save to a dataset
       name -- name of dataset
    """
    name = self._datasetNameWithWorkspace(name)
    self._save(name)
    return name
